/** @file sys_main.c 
*   @brief Application main file
*   @date 03.Apr.2015
*   @version 04.04.00
*
*   This file contains an empty main function,
*   which can be used for the application.
*/

/* 
* Copyright (C) 2009-2015 Texas Instruments Incorporated - www.ti.com 
* 
* 
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*
*    Redistributions of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
*    documentation and/or other materials provided with the   
*    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/


/* USER CODE BEGIN (0) */
/* USER CODE END */

/* Include Files */

#include "sys_common.h"

/* USER CODE BEGIN (1) */
#include "rti.h"
#include "sci.h"
#include "spi.h"
#include "mibspi.h"
#include "sys_dma.h"
/* USER CODE END */

/** @fn void main(void)
*   @brief Application main function
*   @note This function is empty by default.
*
*   This function is called after startup.
*   The user can use this function to implement the application.
*/

/* USER CODE BEGIN (2) */
volatile unsigned long milliseconds,time_out=0;
void loadDataPattern1(uint32 psize, uint16* pptr);
void loadDataPattern2(uint32 psize, uint16* pptr);
void mibspiDmaConfig(mibspiBASE_t *mibspi,uint32 channel, uint32 txchannel, uint32 rxchannel);
/* example data Pattern configuration */
#define D_SIZE      16
uint16 MIBSPI1_TXDATA[D_SIZE]= {0};						// MIBSPI1 transmit buffer in sys ram
uint16 MIBSPI1_RXDATA[D_SIZE]= {0};                 				// MIBSPI1 receive  buffer in sys ram
uint16 SPI4_TX_Data[D_SIZE]= {0};						// SPI4 transmit buffer in sys ram
uint16 SPI4_RX_Data[D_SIZE]= {0};                 				// SPI4 receive  buffer in sys ram
g_dmaCTRL g_dmaCTRLPKT1;							// dma control packet configuration stack for transmit
g_dmaCTRL g_dmaCTRLPKT2;							// dma control packet configuration stack for receive 
/* USER CODE END */

void main(void)
{
/* USER CODE BEGIN (3) */
	spiDAT1_t dataconfig1_t;

	dataconfig1_t.CS_HOLD = FALSE;
	dataconfig1_t.WDEL    = TRUE;
	dataconfig1_t.DFSEL   = SPI_FMT_0;
	dataconfig1_t.CSNR    = 0xFE;

	/** - Initialize LIN/SCI2 Routines to receive Command and transmit data */
	sciInit();
	printf("Terminal connected \r\n");
	/* Initialize RTI driver */
	printf("Initializing RTI\r\n");
	rtiInit();
	/* Enable RTI Compare 0 interrupt notification */
	printf("Enabling RTI Notification\r\n");
	rtiEnableNotification(rtiNOTIFICATION_COMPARE0);      			//500ms tick
	/* Start RTI Counter Block 0 */
	printf("Starting RTI Counter 0\r\n");
	rtiStartCounter(rtiCOUNTER_BLOCK0);
	printf("Enabling CPU IRQ\r\n");

	printf("Entering while loop\r\n");

	/* - creating a data chunk in system ram to start with ... */
	loadDataPattern1(D_SIZE,MIBSPI1_TXDATA);
	loadDataPattern2(D_SIZE,SPI4_TX_Data);
	/* - initializing spi2 */
	spiInit();
	/* - initializing mibspi - enabling tg 0 , length 8 (halcogen file)*/
	mibspiInit();

	/* - configuring dma control packets   */
	g_dmaCTRLPKT1.SADD		= (uint32)MIBSPI1_TXDATA;		// source address		
	g_dmaCTRLPKT1.DADD		= (uint32)&(mibspiRAM1->tx[0].data);	// destination  address		
	g_dmaCTRLPKT1.CHCTRL	= 0;						// channel control		
	g_dmaCTRLPKT1.FRCNT		= 1;					// frame count			
	g_dmaCTRLPKT1.ELCNT		= D_SIZE;				// element count		
	g_dmaCTRLPKT1.ELDOFFSET = 4;						// element destination offset	
	g_dmaCTRLPKT1.ELSOFFSET = 0;						// element source offset	
	g_dmaCTRLPKT1.FRDOFFSET = 0;						// frame destination offset	
	g_dmaCTRLPKT1.FRSOFFSET = 0;						// frame source offset		
	g_dmaCTRLPKT1.PORTASGN  = 4;						// PortB			
	g_dmaCTRLPKT1.RDSIZE    = ACCESS_16_BIT;				// read size			
	g_dmaCTRLPKT1.WRSIZE    = ACCESS_16_BIT;				// write size			
	g_dmaCTRLPKT1.TTYPE     = FRAME_TRANSFER;				// transfer type		
	g_dmaCTRLPKT1.ADDMODERD = ADDR_INC1;					// address mode read		
	g_dmaCTRLPKT1.ADDMODEWR = ADDR_OFFSET;					// address mode write		
	g_dmaCTRLPKT1.AUTOINIT  = AUTOINIT_ON;					// autoinit			

	g_dmaCTRLPKT2.SADD      = (uint32)&(mibspiRAM1->rx[0].data);		// source address             	
	g_dmaCTRLPKT2.DADD      = (uint32)MIBSPI1_RXDATA;			// destination  address       	
	g_dmaCTRLPKT2.CHCTRL    = 0;						// channel control            	
	g_dmaCTRLPKT2.FRCNT	    = 1;					// frame count                	
	g_dmaCTRLPKT2.ELCNT     = D_SIZE;					// element count              	
	g_dmaCTRLPKT2.ELDOFFSET = 0;						// element destination offset 	
	g_dmaCTRLPKT2.ELSOFFSET = 4;						// element destination offset 	
	g_dmaCTRLPKT2.FRDOFFSET = 0;						// frame destination offset   	
	g_dmaCTRLPKT2.FRSOFFSET = 0;						// frame destination offset   	
	g_dmaCTRLPKT2.PORTASGN  = 4;						// PortB			
	g_dmaCTRLPKT2.RDSIZE    = ACCESS_16_BIT;				// read size                  	
	g_dmaCTRLPKT2.WRSIZE    = ACCESS_16_BIT;				// write size                 	
	g_dmaCTRLPKT2.TTYPE     = FRAME_TRANSFER;				// transfer type              	
	g_dmaCTRLPKT2.ADDMODERD = ADDR_OFFSET;					// address mode read          	
	g_dmaCTRLPKT2.ADDMODEWR = ADDR_INC1;					// address mode write         	
	g_dmaCTRLPKT2.AUTOINIT  = AUTOINIT_ON;					// autoinit                   	


   /* upto 32 control packets are supported. */
   /* - setting dma control packets */
   dmaSetCtrlPacket(DMA_CH0,g_dmaCTRLPKT1);					// RAM to MIBSPIRAM
   dmaSetCtrlPacket(DMA_CH1,g_dmaCTRLPKT2);					// MIBSPIRAM to RAM

//   dmaReqAssign(0,1);								// MIBSPI_DMAREQ[0] is connected to reqline 1.
   	   	   	   	   	   	   	   	   	   	//Map Reqline 1 to channel 0

   /* - setting the dma channel to trigger on h/w request */
   dmaSetChEnable(DMA_CH0,DMA_HW); 						// Enable channel 0 on hardware event
   dmaSetChEnable(DMA_CH1,DMA_HW); 						// Enable channel 1 on hardware event

   dmaEnableInterrupt(DMA_CH1,FTC);

   /* - configuring the mibspi dma , channel 0 , tx line -0 , rxline -1     */
   /* - refer to the device data sheet dma request source for mibspi tx/rx  */
   mibspiDmaConfig(mibspiREG1,0,1,0);						// Channel 0
   	   	   	   	   	   	   	   	   	   	// TX 0
   	   	   	   	   	   	   	   	   	   	// RX 1

   dmaEnable();
   _enable_IRQ();

   /* Initiate SPI4 Receive through Interrupt Mode */
//   spiSendAndGetData(spiREG2, &dataconfig1_t, D_SIZE, SPI4_TX_Data, SPI4_RX_Data);

   /* - start the mibspi transfer tg 0 */
//   mibspiTransfer(mibspiREG1,0 );

	while(1)
	{
		if(time_out)
		{
			time_out=0;
			spiSendAndGetData(spiREG4, &dataconfig1_t, D_SIZE, SPI4_TX_Data, SPI4_RX_Data);
			mibspiTransfer(mibspiREG1,0 );
		}
	}
/* USER CODE END */
}

/* USER CODE BEGIN (4) */
void loadDataPattern1(uint32 psize, uint16* pptr)
{
	int i;
	for(i=0;i<psize;i++)
	{
		pptr[i] = i;
	}
 }
void loadDataPattern2(uint32 psize, uint16* pptr)
{
	int i;
	for(i=0;i<psize;i++)
	{
		pptr[i] = psize-1-i;
	}
 }

void mibspiDmaConfig(mibspiBASE_t *mibspi,uint32 channel, uint32 txchannel, uint32 rxchannel)
{
	uint32 bufid  = D_SIZE - 1;
	uint32 icount = 0;

	/* setting transmit and receive channels */
	mibspi->DMACTRL[channel] |= (((rxchannel<<4)|txchannel) << 16);

	/* enabling transmit dma */
	mibspi->DMACTRL[channel] |=  0x00008000;							// RX DMA Enable
	mibspi->DMACTRL[channel] |=  0x00004000;							// TX DMA Enable

	/* setting Initial Count of DMA transfers and the buffer utilized for DMA transfer */
	mibspi->DMACTRL[channel] |=  (icount << 8) |(bufid<<24);

}


/* USER CODE END */
